@@ -8,6 +8,7 @@ if TYPE_CHECKING:
     from .dataset.dataset import DatamintDataset as Dataset
     from .apihandler.api_handler import APIHandler
     from .experiment import Experiment
+    from .api.client import Api
 else:
     import lazy_loader as lazy
 
@@ -19,6 +20,7 @@ else:
             "dataset": ['Dataset'],
             "apihandler.api_handler": ["APIHandler"],
             "experiment": ["Experiment"],
+            "api.client": ["Api"],
         },
     )
 
