@@ -6,12 +6,14 @@ from requests.exceptions import HTTPError
 import logging
 import asyncio
 import aiohttp
-from medimgkit.dicom_utils import anonymize_dicom, to_bytesio, is_dicom
-from medimgkit import dicom_utils
+from medimgkit.dicom_utils import anonymize_dicom, to_bytesio, is_dicom, is_dicom_report, GeneratorWithLength
+from medimgkit import dicom_utils, standardize_mimetype
+from medimgkit.io_utils import is_io_object, peek
+from medimgkit.format_detection import guess_typez, guess_extension, DEFAULT_MIME_TYPE
+from medimgkit.nifti_utils import DEFAULT_NIFTI_MIME, NIFTI_MIMES
 import pydicom
 from pathlib import Path
 from datetime import date
-import mimetypes
 from PIL import Image
 import cv2
 from nibabel.filebasedimages import FileBasedImage as nib_FileBasedImage
@@ -26,13 +28,6 @@ _LOGGER = logging.getLogger(__name__)
 _USER_LOGGER = logging.getLogger('user_logger')
 
 
-def _is_io_object(obj):
-    """
-    Check if an object is a file-like object.
-    """
-    return callable(getattr(obj, "read", None))
-
-
 def _infinite_gen(x):
     while True:
         yield x
@@ -65,7 +60,7 @@ class RootAPIHandler(BaseAPIHandler):
                                             publish: bool = False,
                                             metadata_file: Optional[str | dict] = None,
                                             ) -> str:
-        if _is_io_object(file_path):
+        if is_io_object(file_path):
             name = file_path.name
         else:
             name = file_path
@@ -91,29 +86,39 @@ class RootAPIHandler(BaseAPIHandler):
             name = new_file_path
             _LOGGER.debug(f"New file path: {name}")
 
-        if mimetype is None:
-            mimetype = mimetypes.guess_type(name)[0]
         is_a_dicom_file = None
         if mimetype is None:
-            is_a_dicom_file = is_dicom(name) or is_dicom(file_path)
-            if is_a_dicom_file:
-                mimetype = 'application/dicom'
-            elif name.endswith('.nii') or name.endswith('.nii.gz'):
-                mimetype = 'application/x-nifti'
+            mimetype_list, ext = guess_typez(file_path, use_magic=True)
+            for mime in mimetype_list:
+                if mime in NIFTI_MIMES:
+                    mimetype = DEFAULT_NIFTI_MIME
+                    break
+            else:
+                if ext == '.nii.gz' or name.lower().endswith('nii.gz'):
+                    mimetype = DEFAULT_NIFTI_MIME
+                else:
+                    mimetype = mimetype_list[-1] if mimetype_list else DEFAULT_MIME_TYPE
 
+        mimetype = standardize_mimetype(mimetype)
         filename = os.path.basename(name)
         _LOGGER.debug(f"File name '{filename}' mimetype: {mimetype}")
 
-        if anonymize:
-            if is_a_dicom_file == True or is_dicom(file_path):
-                ds = pydicom.dcmread(file_path)
+        if is_a_dicom_file == True or is_dicom(file_path):
+            if tags is None:
+                tags = []
+            else:
+                tags = list(tags)
+            ds = pydicom.dcmread(file_path)
+            if anonymize:
                 _LOGGER.info(f"Anonymizing {file_path}")
                 ds = anonymize_dicom(ds, retain_codes=anonymize_retain_codes)
-                # make the dicom `ds` object a file-like object in order to avoid unnecessary disk writes
-                f = to_bytesio(ds, name)
-            else:
-                _LOGGER.warning(f"File {file_path} is not a dicom file. Skipping anonymization.")
-                f = _open_io(file_path)
+            lat = dicom_utils.get_dicom_laterality(ds)
+            if lat == 'L':
+                tags.append("left")
+            elif lat == 'R':
+                tags.append("right")
+            # make the dicom `ds` object a file-like object in order to avoid unnecessary disk writes
+            f = to_bytesio(ds, name)
         else:
             f = _open_io(file_path)
 
@@ -180,9 +185,7 @@ class RootAPIHandler(BaseAPIHandler):
             resp_data = await self._run_request_async(request_params, session)
             if 'error' in resp_data:
                 raise DatamintException(resp_data['error'])
-            _LOGGER.info(f"Response on uploading {name}: {resp_data}")
-
-            _USER_LOGGER.info(f'"{name}" uploaded')
+            _LOGGER.debug(f"Response on uploading {name}: {resp_data}")
             return resp_data['id']
         except Exception as e:
             if 'name' in locals():
@@ -207,6 +210,7 @@ class RootAPIHandler(BaseAPIHandler):
                                       segmentation_files: Optional[list[dict]] = None,
                                       transpose_segmentation: bool = False,
                                       metadata_files: Optional[list[str | dict | None]] = None,
+                                      progress_bar: tqdm | None = None,
                                       ) -> list[str]:
         if on_error not in ['raise', 'skip']:
             raise ValueError("on_error must be either 'raise' or 'skip'")
@@ -218,7 +222,10 @@ class RootAPIHandler(BaseAPIHandler):
             metadata_files = _infinite_gen(None)
 
         async with aiohttp.ClientSession() as session:
-            async def __upload_single_resource(file_path, segfiles: dict[str, list | dict], metadata_file: str | dict | None):
+            async def __upload_single_resource(file_path, segfiles: dict[str, list | dict],
+                                               metadata_file: str | dict | None):
+                name = file_path.name if is_io_object(file_path) else file_path
+                name = os.path.basename(name)
                 rid = await self._upload_single_resource_async(
                     file_path=file_path,
                     mimetype=mimetype,
@@ -232,6 +239,12 @@ class RootAPIHandler(BaseAPIHandler):
                     publish=publish,
                     metadata_file=metadata_file,
                 )
+                if progress_bar:
+                    progress_bar.update(1)
+                    progress_bar.set_postfix(file=name)
+                else:
+                    _USER_LOGGER.info(f'"{name}" uploaded')
+
                 if segfiles is not None:
                     fpaths = segfiles['files']
                     names = segfiles.get('names', _infinite_gen(None))
@@ -239,44 +252,69 @@ class RootAPIHandler(BaseAPIHandler):
                         names = _infinite_gen(names)
                     frame_indices = segfiles.get('frame_index', _infinite_gen(None))
                     for f, name, frame_index in tqdm(zip(fpaths, names, frame_indices),
-                                                        desc=f"Uploading segmentations for {file_path}",
-                                                        total=len(fpaths)):
+                                                     desc=f"Uploading segmentations for {file_path}",
+                                                     total=len(fpaths)):
                         if f is not None:
                             await self._upload_segmentations_async(rid,
-                                                                    file_path=f,
-                                                                    name=name,
-                                                                    frame_index=frame_index,
-                                                                    transpose_segmentation=transpose_segmentation)
+                                                                   file_path=f,
+                                                                   name=name,
+                                                                   frame_index=frame_index,
+                                                                   transpose_segmentation=transpose_segmentation)
                 return rid
 
             tasks = [__upload_single_resource(f, segfiles, metadata_file)
                      for f, segfiles, metadata_file in zip(files_path, segmentation_files, metadata_files)]
             return await asyncio.gather(*tasks, return_exceptions=on_error == 'skip')
 
-    def _assemble_dicoms(self, files_path: Sequence[str | IO]) -> tuple[Sequence[str | IO], bool]:
+    def _assemble_dicoms(self, files_path: Sequence[str | IO]
+                         ) -> tuple[Sequence[str | IO], bool, Sequence[int]]:
+        """
+        Assembles DICOM files into a single file.
+
+        Args:
+            files_path: The paths to the DICOM files to assemble.
+
+        Returns:
+            A tuple containing:
+                - The paths to the assembled DICOM files.
+                - A boolean indicating whether the assembly was successful.
+                - same length as the output assembled DICOMs, mapping assembled DICOM to original DICOMs.
+        """
         dicoms_files_path = []
         other_files_path = []
-        for f in files_path:
+        dicom_original_idxs = []
+        others_original_idxs = []
+        for i, f in enumerate(files_path):
             if is_dicom(f):
                 dicoms_files_path.append(f)
+                dicom_original_idxs.append(i)
             else:
                 other_files_path.append(f)
+                others_original_idxs.append(i)
 
         orig_len = len(dicoms_files_path)
         if orig_len == 0:
             _LOGGER.debug("No DICOM files found to assemble.")
-            return files_path, False
+            return files_path, False, []
         dicoms_files_path = dicom_utils.assemble_dicoms(dicoms_files_path, return_as_IO=True)
 
         new_len = len(dicoms_files_path)
         if new_len != orig_len:
             _LOGGER.info(f"Assembled {new_len} dicom files out of {orig_len} files.")
-            files_path = itertools.chain(dicoms_files_path, other_files_path)
+            mapping_idx = [None] * len(files_path)
+            
+            files_path = GeneratorWithLength(itertools.chain(dicoms_files_path, other_files_path), 
+                                             length=new_len + len(other_files_path))
             assembled = True
+            for orig_idx, value in zip(dicom_original_idxs, dicoms_files_path.inverse_mapping_idx):
+                mapping_idx[orig_idx] = value
+            for i, orig_idx in enumerate(others_original_idxs):
+                mapping_idx[orig_idx] = new_len + i
         else:
             assembled = False
+            mapping_idx = [i for i in range(len(files_path))]
 
-        return files_path, assembled
+        return files_path, assembled, mapping_idx
 
     def upload_resource(self,
                         file_path: str | IO | pydicom.dataset.Dataset,
@@ -359,7 +397,8 @@ class RootAPIHandler(BaseAPIHandler):
             transpose_segmentation=transpose_segmentation,
             modality=modality,
             assemble_dicoms=assemble_dicoms,
-            metadata=metadata
+            metadata=metadata,
+            progress_bar=False
         )
 
         return result[0]
@@ -379,7 +418,9 @@ class RootAPIHandler(BaseAPIHandler):
                          transpose_segmentation: bool = False,
                          modality: Optional[str] = None,
                          assemble_dicoms: bool = True,
-                         metadata: list[str | dict | None] | dict | str | None = None
+                         metadata: list[str | dict | None] | dict | str | None = None,
+                         discard_dicom_reports: bool = True,
+                         progress_bar: bool = False
                          ) -> list[str | Exception] | str | Exception:
         """
         Upload resources.
@@ -420,6 +461,27 @@ class RootAPIHandler(BaseAPIHandler):
             raise ValueError("on_error must be either 'raise' or 'skip'")
 
         files_path, is_multiple_resources = RootAPIHandler.__process_files_parameter(files_path)
+
+        # Discard DICOM reports
+        if discard_dicom_reports:
+            old_size = len(files_path)
+            # Create filtered lists maintaining index correspondence
+            filtered_files = []
+            filtered_metadata = []
+
+            for i, f in enumerate(files_path):
+                if not is_dicom_report(f):
+                    filtered_files.append(f)
+                    if metadata is not None:
+                        filtered_metadata.append(metadata[i])
+
+            files_path = filtered_files
+            if metadata is not None:
+                metadata = filtered_metadata
+
+            if old_size is not None and old_size != len(files_path):
+                _LOGGER.info(f"Discarded {old_size - len(files_path)} DICOM report files from upload.")
+
         if isinstance(metadata, (str, dict)):
             _LOGGER.debug("Converting metadatas to a list")
             metadata = [metadata]
@@ -427,8 +489,15 @@ class RootAPIHandler(BaseAPIHandler):
         if metadata is not None and len(metadata) != len(files_path):
             raise ValueError("The number of metadata files must match the number of resources.")
         if assemble_dicoms:
-            files_path, assembled = self._assemble_dicoms(files_path)
+            files_path, assembled, mapping_idx = self._assemble_dicoms(files_path)
             assemble_dicoms = assembled
+        else:
+            mapping_idx = [i for i in range(len(files_path))]
+        n_files = len(files_path)
+
+        if n_files <= 1:
+            # Disable progress bar for single file uploads
+            progress_bar = False
 
         if segmentation_files is not None:
             if assemble_dicoms:
@@ -457,22 +526,32 @@ class RootAPIHandler(BaseAPIHandler):
                             "segmentation_files['names'] must have the same length as segmentation_files['files'].")
 
         loop = asyncio.get_event_loop()
-        task = self._upload_resources_async(files_path=files_path,
-                                            mimetype=mimetype,
-                                            anonymize=anonymize,
-                                            anonymize_retain_codes=anonymize_retain_codes,
-                                            on_error=on_error,
-                                            tags=tags,
-                                            mung_filename=mung_filename,
-                                            channel=channel,
-                                            publish=publish,
-                                            segmentation_files=segmentation_files,
-                                            transpose_segmentation=transpose_segmentation,
-                                            modality=modality,
-                                            metadata_files=metadata,
-                                            )
+        pbar = None
+        try:
+            if progress_bar:
+                pbar = tqdm(total=n_files, desc="Uploading resources", unit="file")
 
-        resource_ids = loop.run_until_complete(task)
+            task = self._upload_resources_async(files_path=files_path,
+                                                mimetype=mimetype,
+                                                anonymize=anonymize,
+                                                anonymize_retain_codes=anonymize_retain_codes,
+                                                on_error=on_error,
+                                                tags=tags,
+                                                mung_filename=mung_filename,
+                                                channel=channel,
+                                                publish=publish,
+                                                segmentation_files=segmentation_files,
+                                                transpose_segmentation=transpose_segmentation,
+                                                modality=modality,
+                                                metadata_files=metadata,
+                                                progress_bar=pbar
+                                                )
+
+            resource_ids = loop.run_until_complete(task)
+        finally:
+            if pbar:
+                pbar.close()
+        
         _LOGGER.info(f"Resources uploaded: {resource_ids}")
 
         if publish_to is not None:
@@ -485,6 +564,10 @@ class RootAPIHandler(BaseAPIHandler):
                 if on_error == 'raise':
                     raise e
 
+        if mapping_idx:
+            _LOGGER.debug(f"Mapping indices for DICOM files: {mapping_idx}")
+            resource_ids = [resource_ids[idx] for idx in mapping_idx]
+
         if is_multiple_resources:
             return resource_ids
         return resource_ids[0]
@@ -563,7 +646,9 @@ class RootAPIHandler(BaseAPIHandler):
         # get the project id by its name
         project = self.get_project_by_name(project_name)
         if 'error' in project:
-            raise ResourceNotFoundError('project', {'project_name': project_name})
+            project = self.get_project_by_id(project_name)
+            if 'error' in project:
+                raise ResourceNotFoundError('project', {'project_name': project_name})
 
         dataset_id = project['dataset_id']
 
@@ -625,7 +710,8 @@ class RootAPIHandler(BaseAPIHandler):
             raise e
 
     @staticmethod
-    def __process_files_parameter(file_path: str | IO | Sequence[str | IO] | pydicom.dataset.Dataset) -> tuple[Sequence[str | IO], bool]:
+    def __process_files_parameter(file_path: str | IO | Sequence[str | IO] | pydicom.dataset.Dataset
+                                  ) -> tuple[Sequence[str | IO], bool]:
         """
         Process the file_path parameter to ensure it is a list of file paths or IO objects.
         """
@@ -640,7 +726,7 @@ class RootAPIHandler(BaseAPIHandler):
                 is_list = False
                 new_file_path = [file_path]
         # Check if is an IO object
-        elif _is_io_object(file_path):
+        elif is_io_object(file_path):
             is_list = False
             new_file_path = [file_path]
         elif not hasattr(file_path, '__len__'):
@@ -690,8 +776,8 @@ class RootAPIHandler(BaseAPIHandler):
 
     def get_resources(self,
                       status: Optional[ResourceStatus] = None,
-                      from_date: Optional[date] = None,
-                      to_date: Optional[date] = None,
+                      from_date: date | str | None = None,
+                      to_date: date | str | None = None,
                       tags: Optional[Sequence[str]] = None,
                       modality: Optional[str] = None,
                       mimetype: Optional[str] = None,
@@ -709,8 +795,8 @@ class RootAPIHandler(BaseAPIHandler):
 
         Args:
             status (ResourceStatus): The resource status. Possible values: 'inbox', 'published', 'archived' or None. If None, it will return all resources.
-            from_date (Optional[date]): The start date.
-            to_date (Optional[date]): The end date.
+            from_date (date | str | None): The start date.
+            to_date (date | str | None): The end date.
             tags (Optional[list[str]]): The tags to filter the resources.
             modality (Optional[str]): The modality of the resources.
             mimetype (Optional[str]): The mimetype of the resources.
@@ -729,9 +815,15 @@ class RootAPIHandler(BaseAPIHandler):
         """
         # Convert datetime objects to ISO format
         if from_date:
-            from_date = from_date.isoformat()
+            if isinstance(from_date, str):
+                date.fromisoformat(from_date)
+            else:
+                from_date = from_date.isoformat()
         if to_date:
-            to_date = to_date.isoformat()
+            if isinstance(to_date, str):
+                date.fromisoformat(to_date)
+            else:
+                to_date = to_date.isoformat()
 
         # Prepare the payload
         payload = {
@@ -831,9 +923,10 @@ class RootAPIHandler(BaseAPIHandler):
 
     async def _async_download_file(self,
                                    resource_id: str,
-                                   save_path: str,
+                                   save_path: str | Path,
                                    session: aiohttp.ClientSession | None = None,
-                                   progress_bar: tqdm | None = None):
+                                   progress_bar: tqdm | None = None,
+                                   add_extension: bool = False) -> str:
         """
         Asynchronously download a file from the server.
 
@@ -842,6 +935,10 @@ class RootAPIHandler(BaseAPIHandler):
             save_path (str): The path to save the file.
             session (aiohttp.ClientSession): The aiohttp session to use for the request.
             progress_bar (tqdm | None): Optional progress bar to update after download completion.
+            add_extension (bool): Whether to add the appropriate file extension based on content type.
+
+        Returns:
+            str: The actual path where the file was saved (important when add_extension=True).
         """
         url = f"{self._get_endpoint_url(RootAPIHandler.ENDPOINT_RESOURCES)}/{resource_id}/file"
         request_params = {
@@ -849,41 +946,90 @@ class RootAPIHandler(BaseAPIHandler):
             'headers': {'accept': 'application/octet-stream'},
             'url': url
         }
+        save_path = str(save_path)  # Ensure save_path is a string for file operations
         try:
             data_bytes = await self._run_request_async(request_params, session, 'content')
-            with open(save_path, 'wb') as f:
-                f.write(data_bytes)
+
+            final_save_path = save_path
+            if add_extension:
+                # Save to temporary file first to determine mimetype from content
+                temp_path = f"{save_path}.tmp"
+                with open(temp_path, 'wb') as f:
+                    f.write(data_bytes)
+
+                # Determine mimetype from file content
+                mimetype_list, ext = guess_typez(temp_path, use_magic=True)
+                mimetype = mimetype_list[-1]
+
+                # get mimetype from resource info if not detected
+                if mimetype is None or mimetype == DEFAULT_MIME_TYPE:
+                    resource_info = self.get_resources_by_ids(resource_id)
+                    mimetype = resource_info.get('mimetype', mimetype)
+
+                # Generate final path with extension if needed
+                if mimetype is not None and mimetype != DEFAULT_MIME_TYPE:
+                    if ext is None:
+                        ext = guess_extension(mimetype)
+                    if ext is not None and not save_path.endswith(ext):
+                        final_save_path = save_path + ext
+
+                # Move file to final location
+                os.rename(temp_path, final_save_path)
+            else:
+                # Standard save without extension detection
+                with open(final_save_path, 'wb') as f:
+                    f.write(data_bytes)
+
             if progress_bar:
                 progress_bar.update(1)
+
+            return final_save_path
+
         except ResourceNotFoundError as e:
             e.set_params('resource', {'resource_id': resource_id})
             raise e
 
     def download_multiple_resources(self,
                                     resource_ids: list[str],
-                                    save_path: list[str] | str
-                                    ) -> None:
+                                    save_path: list[str] | str,
+                                    add_extension: bool = False,
+                                    ) -> list[str]:
         """
         Download multiple resources and save them to the specified paths.
-        
+
         Args:
             resource_ids (list[str]): A list of resource unique ids.
             save_path (list[str] | str): A list of paths to save the files or a directory path.
+            add_extension (bool): Whether to add the appropriate file extension to the save_path based on the content type.
+
+        Returns:
+            list[str]: A list of paths where the files were saved. Important if `add_extension=True`.
         """
+        if isinstance(resource_ids, str):
+            raise ValueError("resource_ids must be a list of strings.")
+
         async def _download_all_async():
             async with aiohttp.ClientSession() as session:
                 tasks = [
-                    self._async_download_file(resource_id, save_path=path, session=session, progress_bar=progress_bar)
+                    self._async_download_file(
+                        resource_id=resource_id,
+                        save_path=path,
+                        session=session,
+                        progress_bar=progress_bar,
+                        add_extension=add_extension
+                    )
                     for resource_id, path in zip(resource_ids, save_path)
                 ]
-                await asyncio.gather(*tasks)
+                return await asyncio.gather(*tasks)
 
         if isinstance(save_path, str):
             save_path = [os.path.join(save_path, r) for r in resource_ids]
 
         with tqdm(total=len(resource_ids), desc="Downloading resources", unit="file") as progress_bar:
             loop = asyncio.get_event_loop()
-            loop.run_until_complete(_download_all_async())
+            final_save_paths = loop.run_until_complete(_download_all_async())
+
+        return final_save_paths
 
     def download_resource_file(self,
                                resource_id: str,
@@ -915,6 +1061,9 @@ class RootAPIHandler(BaseAPIHandler):
             >>> api_handler.download_resource_file('resource_id', save_path='path/to/dicomfile.dcm')
                 saves the file in the specified path.
         """
+        if save_path is None and add_extension:
+            raise ValueError("If add_extension is True, save_path must be provided.")
+
         url = f"{self._get_endpoint_url(RootAPIHandler.ENDPOINT_RESOURCES)}/{resource_id}/file"
         request_params = {'method': 'GET',
                           'headers': {'accept': 'application/octet-stream'},
@@ -924,9 +1073,14 @@ class RootAPIHandler(BaseAPIHandler):
 
             # Get mimetype if needed for auto_convert or add_extension
             mimetype = None
+            mimetype_list = []
+            ext = None
             if auto_convert or add_extension:
-                resource_info = self.get_resources_by_ids(resource_id)
-                mimetype = resource_info['mimetype']
+                mimetype_list, ext = guess_typez(response.content)
+                mimetype = mimetype_list[-1]
+                if mimetype is None or mimetype == DEFAULT_MIME_TYPE:
+                    resource_info = self.get_resources_by_ids(resource_id)
+                    mimetype = resource_info.get('mimetype', None)
 
             if auto_convert:
                 try:
@@ -947,15 +1101,16 @@ class RootAPIHandler(BaseAPIHandler):
             raise e
 
         if save_path is not None:
-            if add_extension:
-                ext = mimetypes.guess_extension(mimetype)
+            if add_extension and mimetype is not None:
+                if ext is None:
+                    ext = guess_extension(mimetype)
                 if ext is not None and not save_path.endswith(ext):
                     save_path += ext
             with open(save_path, 'wb') as f:
                 f.write(response.content)
 
-        if add_extension:
-            return resource_file, save_path
+            if add_extension:
+                return resource_file, save_path
         return resource_file
 
     def download_resource_frame(self,
@@ -1016,16 +1171,45 @@ class RootAPIHandler(BaseAPIHandler):
         """
         if isinstance(resource_ids, str):
             resource_ids = [resource_ids]
-        for rid in resource_ids:
-            url = f"{self._get_endpoint_url(RootAPIHandler.ENDPOINT_RESOURCES)}/{rid}"
-            request_params = {'method': 'DELETE',
-                              'url': url
-                              }
-            try:
-                self._run_request(request_params)
-            except ResourceNotFoundError as e:
-                e.set_params('resource', {'resource_id': rid})
-                raise e
+
+        async def _delete_all_resources_async():
+            async with aiohttp.ClientSession() as session:
+                tasks = [
+                    self._delete_resource_async(resource_id, session)
+                    for resource_id in resource_ids
+                ]
+                await asyncio.gather(*tasks)
+
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(_delete_all_resources_async())
+
+    async def _delete_resource_async(self,
+                                     resource_id: str,
+                                     session: aiohttp.ClientSession | None = None) -> None:
+        """
+        Asynchronously delete a resource by its unique id.
+
+        Args:
+            resource_id (str): The resource unique id.
+            session (aiohttp.ClientSession | None): The aiohttp session to use for the request.
+
+        Raises:
+            ResourceNotFoundError: If the resource does not exist.
+        """
+        if session is not None and not isinstance(session, aiohttp.ClientSession):
+            raise ValueError("session must be an aiohttp.ClientSession object.")
+
+        url = f"{self._get_endpoint_url(RootAPIHandler.ENDPOINT_RESOURCES)}/{resource_id}"
+        request_params = {
+            'method': 'DELETE',
+            'url': url
+        }
+
+        try:
+            await self._run_request_async(request_params, session)
+        except ResourceNotFoundError as e:
+            e.set_params('resource', {'resource_id': resource_id})
+            raise e
 
     def get_datasets(self) -> list[dict]:
         """
