@@ -1,28 +1,99 @@
+from datamint.exceptions import DatamintException
 import argparse
-from datamint.apihandler.api_handler import APIHandler
+# from datamint.apihandler.api_handler import APIHandler
+from datamint import Api
 import os
 from humanize import naturalsize
 import logging
 from pathlib import Path
 import sys
-from medimgkit.dicom_utils import is_dicom
+from medimgkit.dicom_utils import is_dicom, detect_dicomdir, parse_dicomdir_files
 import fnmatch
 from typing import Generator, Optional, Any
 from collections import defaultdict
 from datamint import __version__ as datamint_version
 from datamint import configs
-from datamint.client_cmd_tools.datamint_config import ask_api_key
-from datamint.utils.logging_utils import load_cmdline_logging_config
+from datamint.utils.logging_utils import load_cmdline_logging_config, ConsoleWrapperHandler
+from rich.console import Console
 import yaml
 from collections.abc import Iterable
 import pandas as pd
+import pydicom.errors
 
 # Create two loggings: one for the user and one for the developer
 _LOGGER = logging.getLogger(__name__)
 _USER_LOGGER = logging.getLogger('user_logger')
+logging.getLogger('pydicom').setLevel(logging.ERROR)
+CONSOLE: Console
 
 MAX_RECURSION_LIMIT = 1000
 
+# Default extensions to exclude when --include-extensions is not specified
+DEFAULT_EXCLUDED_EXTENSIONS = [
+    '.txt', '.json', '.xml', '.docx', '.doc', '.pdf', '.xlsx', '.xls', '.csv', '.tsv',
+    '.log', '.ini', '.cfg', '.conf', '.yaml', '.yml', '.md', '.rst', '.html', '.htm',
+    '.exe', '.bat', '.sh', '.py', '.js', '.css',
+    '.sql', '.bak', '.tmp', '.temp', '.lock', '.DS_Store', '.gitignore'
+]
+
+
+def _get_minimal_distinguishing_paths(file_paths: list[str]) -> dict[str, str]:
+    """
+    Generate minimal distinguishing paths for files to avoid ambiguity when multiple files have the same name.
+
+    Args:
+        file_paths: List of file paths
+
+    Returns:
+        Dictionary mapping full path to minimal distinguishing path
+    """
+    if not file_paths:
+        return {}
+
+    # Convert to Path objects and get absolute paths
+    paths = [Path(fp).resolve() for fp in file_paths]
+    result = {}
+
+    # Group files by basename
+    basename_groups = defaultdict(list)
+    for i, path in enumerate(paths):
+        basename_groups[path.name].append((i, path))
+
+    for basename, path_list in basename_groups.items():
+        if len(path_list) == 1:
+            # Only one file with this name, use just the basename
+            idx, path = path_list[0]
+            result[file_paths[idx]] = basename
+        else:
+            # Multiple files with same name, need to distinguish them
+            path_parts_list = [path.parts for _, path in path_list]
+
+            # Find the minimum number of parent directories needed to distinguish
+            max_depth_needed = 1
+            for depth in range(1, max(len(parts) for parts in path_parts_list) + 1):
+                # Check if this depth is enough to distinguish all files
+                suffixes = []
+                for parts in path_parts_list:
+                    if depth >= len(parts):
+                        suffixes.append('/'.join(parts))
+                    else:
+                        suffixes.append('/'.join(parts[-depth:]))
+
+                if len(set(suffixes)) == len(suffixes):
+                    # All suffixes are unique at this depth
+                    max_depth_needed = depth
+                    break
+
+            # Apply the minimal distinguishing paths
+            for (idx, path), parts in zip(path_list, path_parts_list):
+                if max_depth_needed >= len(parts):
+                    distinguishing_path = '/'.join(parts)
+                else:
+                    distinguishing_path = '/'.join(parts[-max_depth_needed:])
+                result[file_paths[idx]] = distinguishing_path
+
+    return result
+
 
 def _read_segmentation_names(segmentation_names_path: str | Path) -> dict:
     """
@@ -35,7 +106,7 @@ def _read_segmentation_names(segmentation_names_path: str | Path) -> dict:
     if segmentation_names_path.suffix in ['.yaml', '.yml']:
         with open(segmentation_names_path, 'r') as f:
             metadata = yaml.safe_load(f)
-    elif segmentation_names_path.suffix in ['.csv', '.tsv']:
+    elif segmentation_names_path.suffix in ['.csv', '.tsv', '.txt']:
         df = pd.read_csv(segmentation_names_path,
                          header=None,
                          index_col=0,
@@ -124,6 +195,22 @@ def walk_to_depth(path: str | Path,
                   depth: int,
                   exclude_pattern: str | None = None) -> Generator[Path, None, None]:
     path = Path(path)
+
+    # Check for DICOMDIR first at current directory level
+    dicomdir_path = detect_dicomdir(path)
+    if dicomdir_path is not None:
+        try:
+            _USER_LOGGER.info(f"Found DICOMDIR file at {path}. Using it as authoritative source for file listing.")
+            dicom_files = parse_dicomdir_files(dicomdir_path)
+            # Yield all DICOM files from DICOMDIR and return early
+            for dicom_file in dicom_files:
+                yield dicom_file
+            return
+        except Exception as e:
+            _USER_LOGGER.warning(f"Failed to parse DICOMDIR at {path}: {e}. Falling back to directory scan.")
+            # Continue with regular directory scanning below
+
+    # Regular directory scanning
     for child in path.iterdir():
         if _is_system_file(child):
             continue
@@ -179,6 +266,7 @@ def handle_api_key() -> str | None:
     If it does not exist, it asks the user to input it.
     Then, it asks the user if he wants to save the API key at a proper location in the machine
     """
+    from datamint.client_cmd_tools.datamint_config import ask_api_key
     api_key = configs.get_value(configs.APIKEY_KEY)
     if api_key is None:
         _USER_LOGGER.info("API key not found. Please provide it:")
@@ -386,6 +474,46 @@ def _collect_metadata_files(files_path: list[str], auto_detect_json: bool) -> tu
     return metadata_files, filtered_files_path
 
 
+def _get_files_from_path(path: str | Path,
+                         recursive_depth: Optional[int] = None,
+                         exclude_pattern: Optional[str] = None,
+                         include_extensions: Optional[list[str]] = None,
+                         exclude_extensions: Optional[list[str]] = None) -> list[str]:
+    """
+    Get files from a path with recursive DICOMDIR detection and parsing.
+
+    Args:
+        path: Path to search for files
+        recursive_depth: Depth for recursive search (None for no recursion, -1 for unlimited)
+        exclude_pattern: Pattern to exclude directories
+        include_extensions: File extensions to include
+        exclude_extensions: File extensions to exclude
+
+    Returns:
+        List of file paths as strings
+    """
+    path = Path(path)
+
+    if path.is_file():
+        return [str(path)]
+
+    try:
+        if recursive_depth is None:
+            recursive_depth = 0
+        elif recursive_depth < 0:
+            recursive_depth = MAX_RECURSION_LIMIT
+        else:
+            recursive_depth = min(MAX_RECURSION_LIMIT, recursive_depth)
+
+        file_paths = walk_to_depth(path, recursive_depth, exclude_pattern)
+        filtered_files = filter_files(file_paths, include_extensions, exclude_extensions)
+        return [str(f) for f in filtered_files]
+
+    except Exception as e:
+        _LOGGER.error(f'Error in recursive search: {e}')
+        raise
+
+
 def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[str]]]:
     parser = argparse.ArgumentParser(
         description='DatamintAPI command line tool for uploading DICOM files and other resources')
@@ -406,6 +534,8 @@ def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[s
     parser.add_argument('--channel', '--name', type=str, required=False,
                         help='Channel name (arbritary) to upload the resources to. \
                             Useful for organizing the resources in the platform.')
+    parser.add_argument('--project', type=str, required=False,
+                        help='Project name to add the uploaded resources to after successful upload.')
     parser.add_argument('--retain-pii', action='store_true', help='Do not anonymize DICOMs')
     parser.add_argument('--retain-attribute', type=_tuple_int_type, action='append',
                         default=[],
@@ -423,7 +553,8 @@ def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[s
                         help='File extensions to be considered for uploading. Default: all file extensions.' +
                         ' Example: --include-extensions dcm jpg png')
     parser.add_argument('--exclude-extensions', type=str, nargs='+',
-                        help='File extensions to be excluded from uploading. Default: none.' +
+                        help='File extensions to be excluded from uploading. ' +
+                        'Default: common non-medical file extensions (.txt, .json, .xml, .docx, etc.) when --include-extensions is not specified.' +
                         ' Example: --exclude-extensions txt csv'
                         )
     parser.add_argument('--segmentation_path', type=_is_valid_path_argparse, metavar="FILE",
@@ -433,7 +564,7 @@ def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[s
                         required=False,
                         help='Path to a yaml or csv file containing the segmentation names.' +
                         ' If yaml, the file may contain two keys: "segmentation_names" and "class_names".'
-                        ' If csv, the file should contain the following columns:'
+                        ' If csv, the file should be in itk-snap label export format, i.e, it should contain the following columns (with no header):'
                         ' index, r, g, b, ..., name')
     parser.add_argument('--yes', action='store_true',
                         help='Automatically answer yes to all prompts')
@@ -463,7 +594,6 @@ def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[s
 
     if args.verbose:
         # Get the console handler and set to debug
-        print(logging.getLogger().handlers)
         logging.getLogger().handlers[0].setLevel(logging.DEBUG)
         logging.getLogger('datamint').setLevel(logging.DEBUG)
         _LOGGER.setLevel(logging.DEBUG)
@@ -476,24 +606,24 @@ def _parse_args() -> tuple[Any, list[str], Optional[list[dict]], Optional[list[s
     if args.include_extensions is not None and args.exclude_extensions is not None:
         raise ValueError("--include-extensions and --exclude-extensions are mutually exclusive.")
 
+    # Apply default excluded extensions if neither include nor exclude extensions are specified
+    if args.include_extensions is None and args.exclude_extensions is None:
+        args.exclude_extensions = DEFAULT_EXCLUDED_EXTENSIONS
+        _LOGGER.debug(f"Applied default excluded extensions: {args.exclude_extensions}")
+
     try:
         if os.path.isfile(args.path):
             file_path = [args.path]
             if args.recursive is not None:
                 _USER_LOGGER.warning("Recursive flag ignored. Specified path is a file.")
         else:
-            try:
-                recursive_depth = 0 if args.recursive is None else args.recursive
-                if recursive_depth < 0:
-                    recursive_depth = MAX_RECURSION_LIMIT
-                else:
-                    recursive_depth = min(MAX_RECURSION_LIMIT, recursive_depth)
-                file_path = walk_to_depth(args.path, recursive_depth, args.exclude)
-                file_path = filter_files(file_path, args.include_extensions, args.exclude_extensions)
-                file_path = list(map(str, file_path))  # from Path to str
-            except Exception as e:
-                _LOGGER.error(f'Error in recursive search: {e}')
-                raise e
+            file_path = _get_files_from_path(
+                path=args.path,
+                recursive_depth=args.recursive,
+                exclude_pattern=args.exclude,
+                include_extensions=args.include_extensions,
+                exclude_extensions=args.exclude_extensions
+            )
 
         if len(file_path) == 0:
             raise ValueError(f"No valid file was found in {args.path}")
@@ -555,12 +685,15 @@ def print_input_summary(files_path: list[str],
     ext_counts = [(ext, count) for ext, count in ext_dict.items()]
     ext_counts.sort(key=lambda x: x[1], reverse=True)
 
+    # Get distinguishing paths for better display
+    distinguishing_paths = _get_minimal_distinguishing_paths(files_path)
+
     _USER_LOGGER.info(f"Number of files to be uploaded: {total_files}")
-    _USER_LOGGER.info(f"\t{files_path[0]}")
+    _USER_LOGGER.info(f"\t{distinguishing_paths[files_path[0]]}")
     if total_files >= 2:
         if total_files >= 3:
             _USER_LOGGER.info("\t(...)")
-        _USER_LOGGER.info(f"\t{files_path[-1]}")
+        _USER_LOGGER.info(f"\t{distinguishing_paths[files_path[-1]]}")
     _USER_LOGGER.info(f"Total size of the upload: {naturalsize(total_size)}")
     _USER_LOGGER.info(f"Number of files per extension:")
     for ext, count in ext_counts:
@@ -602,22 +735,28 @@ def print_results_summary(files_path: list[str],
                           results: list[str | Exception]) -> int:
     # Check for failed uploads
     failure_files = [f for f, r in zip(files_path, results) if isinstance(r, Exception)]
+    # Get distinguishing paths for better error reporting
+    distinguishing_paths = _get_minimal_distinguishing_paths(files_path)
+
     _USER_LOGGER.info(f"\nUpload summary:")
     _USER_LOGGER.info(f"\tTotal files: {len(files_path)}")
     _USER_LOGGER.info(f"\tSuccessful uploads: {len(files_path) - len(failure_files)}")
-    _USER_LOGGER.info(f"\tFailed uploads: {len(failure_files)}")
     if len(failure_files) > 0:
-        _USER_LOGGER.warning(f"\tFailed files: {[os.path.basename(f) for f in failure_files]}")
+        _USER_LOGGER.warning(f"\tFailed uploads: {len(failure_files)}")
+        _USER_LOGGER.warning(f"\tFailed files: {[distinguishing_paths[f] for f in failure_files]}")
         _USER_LOGGER.warning(f"\nFailures:")
         for f, r in zip(files_path, results):
-            _LOGGER.debug(f"Failure: {f} - {r}")
             if isinstance(r, Exception):
-                _USER_LOGGER.warning(f"\t{os.path.basename(f)}: {r}")
+                _USER_LOGGER.warning(f"\t{distinguishing_paths[f]}: {r}")
+    else:
+        CONSOLE.print(f'✅ All uploads successful!', style='success')
     return len(failure_files)
 
 
 def main():
+    global CONSOLE
     load_cmdline_logging_config()
+    CONSOLE = [h for h in _USER_LOGGER.handlers if isinstance(h, ConsoleWrapperHandler)][0].console
 
     try:
         args, files_path, segfiles, metadata_files = _parse_args()
@@ -625,40 +764,55 @@ def main():
         _USER_LOGGER.error(f'Error validating arguments. {e}')
         sys.exit(1)
 
-    print_input_summary(files_path,
-                        args=args,
-                        segfiles=segfiles,
-                        metadata_files=metadata_files,
-                        include_extensions=args.include_extensions)
+    try:
+        print_input_summary(files_path,
+                            args=args,
+                            segfiles=segfiles,
+                            metadata_files=metadata_files,
+                            include_extensions=args.include_extensions)
 
-    if not args.yes:
-        confirmation = input("Do you want to proceed with the upload? (y/n): ")
-        if confirmation.lower() != "y":
-            _USER_LOGGER.info("Upload cancelled.")
+        if not args.yes:
+            confirmation = input("Do you want to proceed with the upload? (y/n): ")
+            if confirmation.lower() != "y":
+                _USER_LOGGER.info("Upload cancelled.")
+                return
+        #######################################
+
+        has_a_dicom_file = any(is_dicom(f) for f in files_path)
+
+        try:
+            api = Api(check_connection=True)
+        except DatamintException as e:
+            _USER_LOGGER.error(f'❌ Connection failed: {e}')
             return
-    #######################################
+        try:
+            print('>>>', segfiles)
+            results = api.resources.upload_resources(channel=args.channel,
+                                                     files_path=files_path,
+                                                     tags=args.tag,
+                                                     on_error='skip',
+                                                     anonymize=args.retain_pii == False and has_a_dicom_file,
+                                                     anonymize_retain_codes=args.retain_attribute,
+                                                     mung_filename=args.mungfilename,
+                                                     publish=args.publish,
+                                                     publish_to=args.project,
+                                                     segmentation_files=segfiles,
+                                                     transpose_segmentation=args.transpose_segmentation,
+                                                     assemble_dicoms=True,
+                                                     metadata=metadata_files,
+                                                     progress_bar=True
+                                                     )
+        except pydicom.errors.InvalidDicomError as e:
+            _USER_LOGGER.error(f'❌ Invalid DICOM file: {e}')
+            return
+        _USER_LOGGER.info('Upload finished!')
+        _LOGGER.debug(f"Number of results: {len(results)}")
 
-    has_a_dicom_file = any(is_dicom(f) for f in files_path)
-
-    api_handler = APIHandler()
-    results = api_handler.upload_resources(channel=args.channel,
-                                           files_path=files_path,
-                                           tags=args.tag,
-                                           on_error='skip',
-                                           anonymize=args.retain_pii == False and has_a_dicom_file,
-                                           anonymize_retain_codes=args.retain_attribute,
-                                           mung_filename=args.mungfilename,
-                                           publish=args.publish,
-                                           segmentation_files=segfiles,
-                                           transpose_segmentation=args.transpose_segmentation,
-                                           assemble_dicoms=True,
-                                           metadata=metadata_files
-                                           )
-    _USER_LOGGER.info('Upload finished!')
-    _LOGGER.debug(f"Number of results: {len(results)}")
-
-    num_failures = print_results_summary(files_path, results)
-    if num_failures > 0:
+        num_failures = print_results_summary(files_path, results)
+        if num_failures > 0:
+            sys.exit(1)
+    except KeyboardInterrupt:
+        CONSOLE.print("\nUpload cancelled by user.", style='warning')
         sys.exit(1)
 
 
