@@ -1,7 +1,7 @@
 import os
 import requests
-from tqdm import tqdm
-from typing import Optional, Callable, Any, Literal
+from tqdm.auto import tqdm
+from typing import Optional, Callable, Any, Literal, Sequence
 import logging
 import shutil
 import json
@@ -13,14 +13,16 @@ from datamint import configs
 from torch.utils.data import DataLoader
 import torch
 from torch import Tensor
-from datamint.apihandler.base_api_handler import DatamintException
+from datamint.exceptions import DatamintException
 from medimgkit.dicom_utils import is_dicom
-from medimgkit.io_utils import read_array_normalized
+from medimgkit.readers import read_array_normalized
+from medimgkit.format_detection import guess_extension, guess_typez
+from medimgkit.nifti_utils import NIFTI_MIMES, get_nifti_shape
 from datetime import datetime
 from pathlib import Path
-from mimetypes import guess_extension
-from datamint.dataset.annotation import Annotation
+from datamint.entities import Annotation, DatasetInfo
 import cv2
+from datamint.entities import Resource
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -64,8 +66,8 @@ class DatamintBaseDataset:
         project_name: str,
         root: str | None = None,
         auto_update: bool = True,
-        api_key: Optional[str] = None,
-        server_url: Optional[str] = None,
+        api_key: str | None = None,
+        server_url: str | None = None,
         return_dicom: bool = False,
         return_metainfo: bool = True,
         return_annotations: bool = True,
@@ -73,14 +75,14 @@ class DatamintBaseDataset:
         include_unannotated: bool = True,
         all_annotations: bool = False,
         # Filtering parameters
-        include_annotators: Optional[list[str]] = None,
-        exclude_annotators: Optional[list[str]] = None,
-        include_segmentation_names: Optional[list[str]] = None,
-        exclude_segmentation_names: Optional[list[str]] = None,
-        include_image_label_names: Optional[list[str]] = None,
-        exclude_image_label_names: Optional[list[str]] = None,
-        include_frame_label_names: Optional[list[str]] = None,
-        exclude_frame_label_names: Optional[list[str]] = None,
+        include_annotators: list[str] | None = None,
+        exclude_annotators: list[str] | None = None,
+        include_segmentation_names: list[str] | None = None,
+        exclude_segmentation_names: list[str] | None = None,
+        include_image_label_names: list[str] | None = None,
+        exclude_image_label_names: list[str] | None = None,
+        include_frame_label_names: list[str] | None = None,
+        exclude_frame_label_names: list[str] | None = None,
     ):
         self._validate_inputs(project_name, include_annotators, exclude_annotators,
                               include_segmentation_names, exclude_segmentation_names,
@@ -104,14 +106,14 @@ class DatamintBaseDataset:
     def _validate_inputs(
         self,
         project_name: str,
-        include_annotators: Optional[list[str]],
-        exclude_annotators: Optional[list[str]],
-        include_segmentation_names: Optional[list[str]],
-        exclude_segmentation_names: Optional[list[str]],
-        include_image_label_names: Optional[list[str]],
-        exclude_image_label_names: Optional[list[str]],
-        include_frame_label_names: Optional[list[str]],
-        exclude_frame_label_names: Optional[list[str]],
+        include_annotators: Sequence[str] | None,
+        exclude_annotators: Sequence[str] | None,
+        include_segmentation_names: Sequence[str] | None,
+        exclude_segmentation_names: Sequence[str] | None,
+        include_image_label_names: Sequence[str] | None,
+        exclude_image_label_names: Sequence[str] | None,
+        include_frame_label_names: Sequence[str] | None,
+        exclude_frame_label_names: Sequence[str] | None,
     ) -> None:
         """Validate input parameters."""
         if project_name is None:
@@ -174,23 +176,12 @@ class DatamintBaseDataset:
 
     def _setup_api_handler(self, server_url: Optional[str], api_key: Optional[str], auto_update: bool) -> None:
         """Setup API handler and validate connection."""
-        from datamint.apihandler.api_handler import APIHandler
-
-        self.api_handler = APIHandler(
-            root_url=server_url,
+        from datamint import Api
+        self.api = Api(
+            server_url=server_url,
             api_key=api_key,
-            check_connection=auto_update
+            check_connection=self.auto_update
         )
-        self.server_url = self.api_handler.root_url
-        self.api_key = self.api_handler.api_key
-
-        if self.api_key is None:
-            _LOGGER.warning(
-                "API key not provided. If you want to download data, please provide an API key, "
-                f"either by passing it as an argument, "
-                f"setting environment variable {configs.ENV_VARS[configs.APIKEY_KEY]} or "
-                "using datamint-config command line tool."
-            )
 
     def _setup_directories(self, root: str | None) -> None:
         """Setup root and dataset directories."""
@@ -234,26 +225,34 @@ class DatamintBaseDataset:
         else:
             self._check_version()
 
+    def _init_metainfo(self) -> None:
+        # get the server info
+        self.project_info = self.get_info()
+        self.metainfo = self._get_datasetinfo().asdict().copy()
+        self.metainfo['updated_at'] = None
+        self.metainfo['resources'] = []
+        self.metainfo['all_annotations'] = self.all_annotations
+        self.images_metainfo = self.metainfo['resources']
+
     def _load_metadata(self) -> bool:
         """Load and process dataset metadata."""
         if hasattr(self, 'metainfo'):
             _LOGGER.warning("Metadata already loaded.")
         metadata_path = os.path.join(self.dataset_dir, 'dataset.json')
         if not os.path.isfile(metadata_path):
-            # get the server info
-            self.project_info = self.get_info()
-            self.metainfo = self._get_datasetinfo().copy()
-            self.metainfo['updated_at'] = None
-            self.metainfo['resources'] = []
-            self.metainfo['all_annotations'] = self.all_annotations
-            self.images_metainfo = self.metainfo['resources']
+            self._init_metainfo()
             return False
         else:
             with open(metadata_path, 'r') as file:
                 self.metainfo = json.load(file)
         self.images_metainfo = self.metainfo['resources']
         # Convert annotations from dict to Annotation objects
-        self._convert_metainfo_to_clsobj()
+        try:
+            self._convert_metainfo_to_clsobj()
+        except Exception as e:
+            _LOGGER.warning(f"Failed to convert annotations. Redownloading dataset. {type(e)}")
+            self._init_metainfo()
+            return False
         return True
 
     def _convert_metainfo_to_clsobj(self):
@@ -283,10 +282,12 @@ class DatamintBaseDataset:
         """Post-process data after loading metadata."""
         self._check_integrity()
         self._calculate_dataset_length()
-        self._precompute_frame_data()
+        if self.return_frame_by_frame:
+            self._precompute_frame_data()
+        self.subset_indices = list(range(self.dataset_length))
         self._setup_labels()
 
-        if self.discard_without_annotations and self.return_frame_by_frame:
+        if self.discard_without_annotations:
             self._filter_unannotated()
 
     def _calculate_dataset_length(self) -> None:
@@ -301,9 +302,8 @@ class DatamintBaseDataset:
 
     def _precompute_frame_data(self) -> None:
         """Precompute frame-related data for efficient indexing."""
-        self.num_frames_per_resource = self.__compute_num_frames_per_resource()
-        self._cumulative_frames = np.cumsum([0] + self.num_frames_per_resource)
-        self.subset_indices = list(range(self.dataset_length))
+        num_frames_per_resource = self.__compute_num_frames_per_resource()
+        self._cumulative_frames = np.cumsum([0] + num_frames_per_resource)
 
     def _setup_labels(self) -> None:
         """Setup label sets and mappings."""
@@ -360,11 +360,13 @@ class DatamintBaseDataset:
     @property
     def segmentation_labels_set(self) -> list[str]:
         """Returns the set of segmentation labels in the dataset."""
-        return self.frame_lsets['segmentation']
+        a = set(self.frame_lsets['segmentation'])
+        b = set(self.image_lsets['segmentation'])
+        return list(a.union(b))
 
     def _get_annotations_internal(
         self,
-        annotations: list[Annotation],
+        annotations: Sequence[Annotation],
         type: Literal['label', 'category', 'segmentation', 'all'] = 'all',
         scope: Literal['frame', 'image', 'all'] = 'all'
     ) -> list[Annotation]:
@@ -411,26 +413,38 @@ class DatamintBaseDataset:
     @staticmethod
     def read_number_of_frames(filepath: str) -> int:
         """Read the number of frames in a file."""
-        if is_dicom(filepath):
+
+        mimetypes, ext = guess_typez(filepath)
+        mimetype = mimetypes[0]
+        if mimetype is None:
+            raise ValueError(f"Could not determine MIME type for file: {filepath}")
+
+        if mimetype == 'application/dicom':
             ds = pydicom.dcmread(filepath)
             return getattr(ds, 'NumberOfFrames', 1)
-        elif filepath.lower().endswith(('.mp4', '.avi')):
+        elif mimetype.startswith('video/'):
             cap = cv2.VideoCapture(filepath)
             try:
                 return int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
             finally:
                 cap.release()
-        elif filepath.lower().endswith(('.png', '.jpg', '.jpeg')):
+        elif mimetype in ('image/png', 'image/jpeg', 'image/jpg', 'image/bmp', 'image/tiff'):
             return 1
+        elif mimetype in NIFTI_MIMES:
+            shape = get_nifti_shape(filepath)
+            if len(shape) == 3:
+                return shape[-1]
+            elif len(shape) > 3:
+                return shape[3]
+            else:
+                return 1
         else:
-            raise ValueError(f"Unsupported file type: {filepath}")
+            raise ValueError(f"Unsupported file type '{mimetype}' for file {filepath}")
 
     def get_resources_ids(self) -> list[str]:
         """Get list of resource IDs."""
-        return [
-            self.__getitem_internal(i, only_load_metainfo=True)['metainfo']['id']
-            for i in self.subset_indices
-        ]
+        return [self._get_image_metainfo(i, bypass_subset_indices=True)['metainfo']['id']
+                for i in self.subset_indices]
 
     def _get_labels_set(self, framed: bool) -> tuple[dict, dict[str, dict[str, int]]]:
         """Returns the set of labels and mappings to integers.
@@ -525,18 +539,18 @@ class DatamintBaseDataset:
         if missing_files:
             raise DatamintDatasetException(f"Image files not found: {missing_files}")
 
-    def _get_datasetinfo(self) -> dict:
+    def _get_datasetinfo(self) -> DatasetInfo:
         """Get dataset information from API."""
         if self._server_dataset_info is not None:
             return self._server_dataset_info
-        all_datasets = self.api_handler.get_datasets()
+        all_datasets = self.api._datasetsinfo.get_all()
 
         for dataset in all_datasets:
-            if dataset['id'] == self.dataset_id:
+            if dataset.id == self.dataset_id:
                 self._server_dataset_info = dataset
                 return dataset
 
-        available_datasets = [(d['name'], d['id']) for d in all_datasets]
+        available_datasets = [(d.name, d.id) for d in all_datasets]
         raise DatamintDatasetException(
             f"Dataset with id '{self.dataset_id}' not found. "
             f"Available datasets: {available_datasets}"
@@ -546,13 +560,12 @@ class DatamintBaseDataset:
         """Get project information from API."""
         if hasattr(self, 'project_info') and self.project_info is not None:
             return self.project_info
-        project = self.api_handler.get_project_by_name(self.project_name)
-        if 'error' in project:
-            available_projects = project['all_projects']
+        project = self.api.projects.get_by_name(self.project_name)
+        if project is None:
             raise DatamintDatasetException(
-                f"Project with name '{self.project_name}' not found. "
-                f"Available projects: {available_projects}"
+                f"Project with name '{self.project_name}' not found."
             )
+        project = project.asdict()
         self.project_info = project
         self.dataset_id = project['dataset_id']
         return project
@@ -591,31 +604,10 @@ class DatamintBaseDataset:
         lines = [head] + [" " * 4 + line for line in body]
         return "\n".join(lines)
 
-    def download_project(self) -> None:
-        """Download project data from API."""
-
-        dataset_info = self._get_datasetinfo()
-        self.dataset_id = dataset_info['id']
-        self.last_updaded_at = dataset_info['updated_at']
-
-        self.api_handler.download_project(
-            self.project_info['id'],
-            self.dataset_zippath,
-            all_annotations=self.all_annotations,
-            include_unannotated=self.include_unannotated
-        )
-
-        _LOGGER.debug("Downloaded dataset")
-
-        if os.path.getsize(self.dataset_zippath) == 0:
-            raise DatamintDatasetException("Download failed.")
-
-        self._extract_and_update_metadata()
-
     def _get_dataset_id(self) -> str:
         if self.dataset_id is None:
             dataset_info = self._get_datasetinfo()
-            self.dataset_id = dataset_info['id']
+            self.dataset_id = dataset_info.id
         return self.dataset_id
 
     def _extract_and_update_metadata(self) -> None:
@@ -637,7 +629,7 @@ class DatamintBaseDataset:
 
         # Save updated metadata
         with open(datasetjson_path, 'w') as file:
-            json.dump(self.metainfo, file, default=lambda o: o.to_dict() if hasattr(o, 'to_dict') else o)
+            json.dump(self.metainfo, file, default=lambda o: o.asdict() if hasattr(o, 'asdict') else o)
 
         self.images_metainfo = self.metainfo['resources']
         # self._convert_metainfo_to_clsobj()
@@ -645,19 +637,19 @@ class DatamintBaseDataset:
     def _update_metadata_timestamps(self) -> None:
         """Update metadata with correct timestamps."""
         if 'updated_at' not in self.metainfo:
-            self.metainfo['updated_at'] = self.last_updaded_at
+            self.metainfo['updated_at'] = self.last_updated_at
         else:
             try:
                 local_time = datetime.fromisoformat(self.metainfo['updated_at'])
-                server_time = datetime.fromisoformat(self.last_updaded_at)
+                server_time = datetime.fromisoformat(self.last_updated_at)
 
                 if local_time < server_time:
                     _LOGGER.warning(
                         f"Inconsistent updated_at dates detected "
-                        f"({self.metainfo['updated_at']} < {self.last_updaded_at}). "
-                        f"Fixing it to {self.last_updaded_at}"
+                        f"({self.metainfo['updated_at']} < {self.last_updated_at}). "
+                        f"Fixing it to {self.last_updated_at}"
                     )
-                    self.metainfo['updated_at'] = self.last_updaded_at
+                    self.metainfo['updated_at'] = self.last_updated_at
             except Exception as e:
                 _LOGGER.warning(f"Failed to parse updated_at date: {e}")
 
@@ -689,6 +681,9 @@ class DatamintBaseDataset:
             img = (img - min_val) / (img.max() - min_val) * 255
             img = img.astype(np.uint8)
 
+        if not img.flags.writeable:
+            img = img.copy()
+
         img_tensor = torch.from_numpy(img).contiguous()
 
         if isinstance(img_tensor, torch.ByteTensor):
@@ -828,7 +823,7 @@ class DatamintBaseDataset:
 
         try:
             external_metadata_info = self._get_datasetinfo()
-            server_updated_at = external_metadata_info['updated_at']
+            server_updated_at = external_metadata_info.updated_at
         except Exception as e:
             _LOGGER.warning(f"Failed to check for updates in {self.project_name}: {e}")
             return
@@ -855,20 +850,21 @@ class DatamintBaseDataset:
             _LOGGER.info('Local version is up to date with the latest version.')
 
     def _fetch_new_resources(self,
-                             all_uptodate_resources: list[dict]) -> list[dict]:
+                             all_uptodate_resources: list[Resource]) -> list[dict]:
         local_resources = self.images_metainfo
         local_resources_ids = [res['id'] for res in local_resources]
         new_resources = []
         for resource in all_uptodate_resources:
+            resource = resource.asdict()
             if resource['id'] not in local_resources_ids:
                 resource['file'] = str(self._get_resource_file_path(resource))
                 resource['annotations'] = []
                 new_resources.append(resource)
         return new_resources
 
-    def _fetch_deleted_resources(self, all_uptodate_resources: list[dict]) -> list[dict]:
+    def _fetch_deleted_resources(self, all_uptodate_resources: list[Resource]) -> list[dict]:
         local_resources = self.images_metainfo
-        all_uptodate_resources_ids = [res['id'] for res in all_uptodate_resources]
+        all_uptodate_resources_ids = [res.id for res in all_uptodate_resources]
         deleted_resources = []
         for resource in local_resources:
             try:
@@ -887,7 +883,7 @@ class DatamintBaseDataset:
         # server_updated_at = external_metadata_info['updated_at']
 
         ### RESOURCES ###
-        all_uptodate_resources = self.api_handler.get_project_resources(self.get_info()['id'])
+        all_uptodate_resources = self.api.projects.get_project_resources(self.get_info()['id'])
         new_resources = self._fetch_new_resources(all_uptodate_resources)
         deleted_resources = self._fetch_deleted_resources(all_uptodate_resources)
 
@@ -897,8 +893,11 @@ class DatamintBaseDataset:
             new_resources_path = [Path(self.dataset_dir) / r['file'] for r in new_resources]
             new_resources_ids = [r['id'] for r in new_resources]
             _LOGGER.info(f"Downloading {len(new_resources)} new resources...")
-            self.api_handler.download_multiple_resources(new_resources_ids,
-                                                        save_path=new_resources_path)
+            new_res_paths = self.api.resources.download_multiple_resources(new_resources_ids,
+                                                                           save_path=new_resources_path,
+                                                                           add_extension=True)
+            for new_rpath, r in zip(new_res_paths, new_resources):
+                r['file'] = str(Path(new_rpath).relative_to(self.dataset_dir))
             _LOGGER.info(f"Downloaded {len(new_resources)} new resources.")
 
         for r in deleted_resources:
@@ -906,16 +905,19 @@ class DatamintBaseDataset:
         ################
 
         ### ANNOTATIONS ###
-        all_annotations = self.api_handler.get_annotations(worklist_id=self.project_info['worklist_id'],
-                                                           status='published' if self.all_annotations else None)
+        _LOGGER.info("Fetching new annotations...")
+        all_annotations = self.api.annotations.get_list(worklist_id=self.project_info['worklist_id'],
+                                                        status=None if self.all_annotations else 'published',
+                                                        load_ai_segmentations=self.all_annotations)
+
         # group annotations by resource ID
-        annotations_by_resource = {}
+        annotations_by_resource: dict[str, list[Annotation]] = {}
         for ann in all_annotations:
             # add the local filepath
             filepath = self._get_annotation_file_path(ann)
             if filepath is not None:
-                ann['file'] = str(filepath)
-            resource_id = ann['resource_id']
+                ann.file = str(filepath)
+            resource_id = ann.resource_id
             if resource_id not in annotations_by_resource:
                 annotations_by_resource[resource_id] = []
             annotations_by_resource[resource_id].append(ann)
@@ -933,11 +935,11 @@ class DatamintBaseDataset:
             # check if segmentation annotations need to be downloaded
             # Also check if annotations need to be deleted
             old_ann_ids = set([ann.id for ann in old_resource_annotations if hasattr(ann, 'id')])
-            new_ann_ids = set([ann['id'] for ann in new_resource_annotations])
+            new_ann_ids = set([ann.id for ann in new_resource_annotations])
 
             # Find annotations to add, update, or remove
             annotations_to_add = [ann for ann in new_resource_annotations
-                                  if ann['id'] not in old_ann_ids]
+                                  if ann.id not in old_ann_ids]
             annotations_to_remove = [ann for ann in old_resource_annotations
                                      if getattr(ann, 'id', 'NA') not in new_ann_ids]
 
@@ -966,33 +968,35 @@ class DatamintBaseDataset:
                     _LOGGER.error(f"Error deleting annotation file {filepath}: {e}")
 
             # Update resource annotations list - convert to Annotation objects
-            resource['annotations'] = [Annotation.from_dict(ann) for ann in new_resource_annotations]
+            # resource['annotations'] = [Annotation.from_dict(ann) for ann in new_resource_annotations]
+            resource['annotations'] = new_resource_annotations
 
         # Batch download all segmentation files
         if segmentations_to_download:
             _LOGGER.info(f"Downloading {len(segmentations_to_download)} segmentation files...")
-            self.api_handler.download_multiple_segmentations(segmentations_to_download, segmentation_paths)
+            self.api.annotations.download_multiple_files(segmentations_to_download, segmentation_paths)
             _LOGGER.info(f"Downloaded {len(segmentations_to_download)} segmentation files.")
 
         ###################
         # update metadata
-        self.metainfo['updated_at'] = self._get_datasetinfo()['updated_at']
+        self.metainfo['updated_at'] = self._get_datasetinfo().updated_at
         self.metainfo['all_annotations'] = self.all_annotations
         # save updated metadata
         datasetjson_path = os.path.join(self.dataset_dir, 'dataset.json')
         with open(datasetjson_path, 'w') as file:
-            json.dump(self.metainfo, file, default=lambda o: o.to_dict() if hasattr(o, 'to_dict') else o)
+            json.dump(self.metainfo, file, default=lambda o: o.asdict() if hasattr(o, 'asdict') else o)
 
     def _get_resource_file_path(self, resource: dict) -> Path:
         """Get the local file path for a resource."""
         if 'file' in resource and resource['file'] is not None:
             return Path(resource['file'])
         else:
-            ext = guess_extension(resource['mimetype'], strict=False)
-            if ext is None:
-                _LOGGER.warning(f"Could not guess extension for resource {resource['id']}.")
-                ext = ''
-            return Path('images', f"{resource['id']}{ext}")
+            # ext = guess_extension(resource['mimetype'])
+            # if ext is None:
+            #     _LOGGER.warning(f"Could not guess extension for resource {resource['id']}.")
+            #     ext = ''
+            # return Path('images', f"{resource['id']}{ext}")
+            return Path('images', resource['id'])
 
     def _get_annotation_file_path(self, annotation: dict | Annotation) -> Path | None:
         """Get the local file path for an annotation."""
